unit urbox;

// __PTC_RESIZE_WINDOW__        Resizable window
// __PTC_ALLOW_CLOSE__          window can be closed
// __PTC_CLOSE_ON_ESCAPE__      Close window when escape is pressed
// __PTC_WINDOWED__
// __PTC_SYSTEM_MENU__
// __PTC_CENTER_WINDOW__
// __PTC_DISABLE_SCREENSAVER__
// __PTC_ICON__                 Show Icon
// __PTC_DDRAW__
// __PTC_GDI__
// __PTC_VFW__

interface

procedure MainFunc;

implementation

uses
  mmsystem,
  windows,
  messages,
{$ifdef __PTC_VFW__}
  vfw;
{$endif}
{$ifdef __PTC_DDRAW__}
  ddraw;
{$endif}
{$ifdef __PTC_DDRAW7__}
  ddraw7;
{$endif}
{$ifdef __PTC_GDI__}
  gdi;
{$endif}

const
  WIDTH:  Integer = 640;
  HEIGHT: Integer = 480;

type
  TQuad = packed record
  case integer of
    0: (Color: Longint);
    1: (b1,b2,b3,b4: byte);
  end;

var
  pixel: array of Longint;

function FloatToStr(Value: Extended): string;
begin
  Result := '';
  if Value > 1000 then
    Result := Result + Char(Ord('0') + Trunc(Value/1000));
  if Value > 100 then begin
    Value := value - 1000 * Trunc(Value/1000);
    Result := Result + Char(Ord('0') + Trunc(Value/100));
  end;
  if Value > 10 then begin
    Value := value - 100 * Trunc(Value/100);
    Result := Result + Char(Ord('0') + Trunc(Value/10));
  end;
  Value := value - 10 * Trunc(Value/10);
  Result := Result + Char(Ord('0') + Trunc(Value)) + '.';
  Value := value - Trunc(Value);
  Value := Value * 10;
  Result := Result + Char(Ord('0') + Trunc(Value));
  Value := value - Trunc(Value);
  Value := Value * 10;
  Result := Result + Char(Ord('0') + Trunc(Value));
end;

function Min(A,B: Integer): Integer;
begin
  if A < B then
    Result := A
  else
    Result := B;
end;

procedure AdditiveLine(x1, y1, x2, y2: double; c : Integer);
var
  tmp: double;
  itmp: integer;
  dfx, dfy: double;
  ix1, ix2, iy1, iy2: Integer;
  x, y: integer;
  dx, dy: integer;
  Quad: ^TQuad;
  r,g,b: integer;
begin
  r := (c shr 16) and $FF;
  g := (c shr 8) and $FF;
  b := c and $FF;

  // normal additive line
  if (x1 > x2) then begin
    tmp := x1; x1 := x2; x2:= tmp;
    tmp := y1; y1 := y2; y2:= tmp;
  end;

  ix1 := trunc(x1);
  ix2 := trunc(x2);
  iy1 := trunc(y1);
  iy2 := trunc(y2);

  if (ix1 = ix2) then begin  // vertical line
    if (iy1 > iy2) then begin
      itmp := iy1; iy1 := iy2; iy2:= itmp;
    end;

    y := iy1;
    Quad := @Pixel[iy1 * width + ix1];
    while (y <> iy2) do begin
      Quad^.b1   := min(255, Quad^.b1 + r);
      Quad^.b2   := min(255, Quad^.b2 + g);
      Quad^.b3   := min(255, Quad^.b3 + b);

      Inc(y);
      Inc(Quad, Width);
    end;

    exit;
  end;

  if (iy1 = iy2) then begin  // horizontal line
    x := ix1;
    Quad := @Pixel[iy1 * width + x];
    while (x <> ix2) do begin
      Quad^.b1   := min(255, Quad^.b1 + r);
      Quad^.b2   := min(255, Quad^.b2 + g);
      Quad^.b3   := min(255, Quad^.b3 + b);

      Inc(x);
      Inc(Quad);
    end;

    Exit;
  end;

  dfx := x2 - x1;
  dfy := y2 - y1;

  if (dfx = 0) or (dfy = 0) then
    Exit;

  if abs(dfx) > abs(dfy) then begin
    x := trunc(x1);
    y := trunc(y1 * 65535);
    dy := trunc(dfy/dfx * 65535);
    if dfy > 0 then begin
      while x <= ix2 do begin
        Quad := @Pixel[(y shr 16) * width + x];
        Quad^.b1   := min(255, Quad^.b1 + r);
        Quad^.b2   := min(255, Quad^.b2 + g);
        Quad^.b3   := min(255, Quad^.b3 + b);

        y := y + dy;
        Inc(x)
      end;
    end else begin
      while x <= ix2 do begin
        Quad := @Pixel[(y shr 16) * width + x];
        Quad^.b1   := min(255, Quad^.b1 + r);
        Quad^.b2   := min(255, Quad^.b2 + g);
        Quad^.b3   := min(255, Quad^.b3 + b);

        y := y + dy;
        Inc(x)
      end;
    end;
  end else begin
    x := trunc(x1 * 65535);
    y := trunc(y1);
    dx := trunc(dfx/dfy * 65535);
    if dfy > 0 then begin
      while y <= iy2 do begin
        Quad := @Pixel[y * width + (x shr 16)];
        Quad^.b1   := min(255, Quad^.b1 + r);
        Quad^.b2   := min(255, Quad^.b2 + g);
        Quad^.b3   := min(255, Quad^.b3 + b);

        x := x + dx;
        Inc(y)
      end;
    end else begin
      while y >= iy2 do begin
        Quad := @Pixel[y * width + (x shr 16)];
        Quad^.b1   := min(255, Quad^.b1 + r);
        Quad^.b2   := min(255, Quad^.b2 + g);
        Quad^.b3   := min(255, Quad^.b3 + b);

        x := x - dx;
        Dec(y)
      end;
    end;
  end;
end;

procedure paintcube;
var
 rx, ry,rz, lag, t, ti: double;
 n, i: integer;
 pb, point: array[0..7,0..2] of double;
 tx,ty,s: double;
 sr,cr: double;
 c: integer;
begin
  fillchar(pixel[0], Width * Height * 4, 0);

  ti := timegettime * 0.00015;

  t := ti + 0.05 * (cos(ti * 0.41 + 0.123) + cos(ti * 0.51 + 0.234) +
                     cos(ti * 0.61 + 0.623) + cos(ti * 0.21 + 0.723));
  lag := 0.005 + 0.4 * 0.005 * (cos(ti * 0.41 + 0.0123) + cos(ti * 0.51 + 0.0234) +
                         cos(ti * 0.61 + 0.0623) + cos(ti * 0.021 + 1.723));
  c := RGB(26, 17, 10);

{
  t := ti + 0.02 * (cos(ti * 0.41 + 0.123) + cos(ti * 0.51 + 0.234) +
                     cos(ti * 0.61 + 0.623) + cos(ti * 0.21 + 0.723));
  lag := 0.05 * (0.9 + cos(ti * 0.41 + 0.0123) + cos(ti * 0.51 + 0.0234) +
                        cos(ti * 0.61 + 0.0623) + cos(ti * 0.021 + 1.723));

  c := RGB(27, 13, 13);
}
  point[0][0] := -1;
  point[0][1] := 1;
  point[0][2] := 1;
  point[1][0] := 1;
  point[1][1] := 1;
  point[1][2] := 1;
  point[2][0] := 1;
  point[2][1] := -1;
  point[2][2] := 1;
  point[3][0] := -1;
  point[3][1] := -1;
  point[3][2] := 1;
  point[4][0] := -1;
  point[4][1] := 1;
  point[4][2] := -1;
  point[5][0] := 1;
  point[5][1] := 1;
  point[5][2] := -1;
  point[6][0] := 1;
  point[6][1] := -1;
  point[6][2] := -1;
  point[7][0] := -1;
  point[7][1] := -1;
  point[7][2] := -1;
  move(point, pb, 8 * 3 * sizeOf(double));

  s := 0;
  for i := 0 to 150 do begin
    move(pb, point, 8 * 3 * sizeOf(double));
    for n := 0 to 7 do begin
      rx := 2.4 * t + 5 * ti;
      ry := 2.5 * t + 4 * ti;
      rz := 2.6 * t;

      tx := point[n][0];
      ty := point[n][1];
      sr := sin(rx);
      cr := cos(rx);
      point[n][0] := cr * tx - sr * ty;
      point[n][1] := sr * tx + cr * ty;

      tx := point[n][0];
      ty := point[n][2];
      sr := sin(ry);
      cr := cos(ry);
      point[n][0] := cr * tx - sr * ty;
      point[n][2] := sr * tx + cr * ty;

      tx := point[n][1];
      ty := point[n][2];
      sr := sin(rz);
      cr := cos(rz);
      point[n][1] := cr * tx - sr * ty;
      point[n][2] := sr * tx + cr * ty;

      point[n][0] := point[n][0] * s;
      point[n][1] := point[n][1] * s;
      point[n][2] := point[n][2] * s;

      point[n][0] := (point[n][0] - point[n][2] * 0.6) * Height * 0.4 + Width div 2;
      point[n][1] := (point[n][1] + point[n][2] * 0.3) * Height * 0.4 + Height div 2;
    end;

    AdditiveLine(point[0][0], point[0][1], point[1][0], point[1][1], c);
    AdditiveLine(point[1][0], point[1][1], point[2][0], point[2][1], c);
    AdditiveLine(point[2][0], point[2][1], point[3][0], point[3][1], c);
    AdditiveLine(point[3][0], point[3][1], point[0][0], point[0][1], c);

    AdditiveLine(point[0][0], point[0][1], point[4][0], point[4][1], c);
    AdditiveLine(point[1][0], point[1][1], point[5][0], point[5][1], c);
    AdditiveLine(point[2][0], point[2][1], point[6][0], point[6][1], c);
    AdditiveLine(point[3][0], point[3][1], point[7][0], point[7][1], c);

    AdditiveLine(point[4][0], point[4][1], point[5][0], point[5][1], c);
    AdditiveLine(point[5][0], point[5][1], point[6][0], point[6][1], c);
    AdditiveLine(point[6][0], point[6][1], point[7][0], point[7][1], c);
    AdditiveLine(point[7][0], point[7][1], point[4][0], point[4][1], c);

    s := s + 0.004;
    t := t - lag;
  end;
end;

procedure MainFunc;
{$ifdef __PTC_WINDOWED__}
var
  NrFrame:   Integer;
  t, startTime: double;
  s: string;
{$endif}
begin
  SetLength(pixel, Width * Height);

{$ifdef __PTC_WINDOWED__}
  startTime := timeGetTime * 0.001;
  NrFrame := 0;
{$endif}

  if (ptc_open('test', WIDTH, HEIGHT) = 0) then
    Exit;
  while (True) do begin

    paintcube;
    ptc_update(@pixel[0]);

{$ifdef __PTC_WINDOWED__}
    Inc(NrFrame);
    t := (timeGetTime * 0.001) - StartTime;
    if t > 2 then begin
      s := 'test ' + FloatToStr(NrFrame/t) + ' fps';

      SendMessage(wnd, WM_SETTEXT, 0, Longint(Pchar(s)));

      startTime := timeGetTime * 0.001;
      NrFrame := 0;
    end;
{$endif}
  end;
  ptc_close;
end;

end.
